#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Import necessary packages.
import argparse
import logging
import os
import os.path as osp
import warnings

from keras import backend as K
from keras.models import load_model
import tensorflow as tf
from tensorflow.python.framework import graph_io
from tensorflow.python.framework.graph_util import convert_variables_to_constants

from config import output_log

# Set necessary environment.
warnings.filterwarnings("ignore")
# Create logger for debugging.
output_log.init_log('./log/update_url')


def freeze_session(session, keep_var_names=None, output_names=None, clear_devices=True):
    """  Freezes the state of a session into a pruned computation graph.

    Creates a new computation graph where variable nodes are replaced by
    constants taking their current value in the session. The new graph will be
    pruned so sub-graphs that are not necessary to compute the requested
    outputs are removed.

    :param session: The TensorFlow session to be frozen.
    :param keep_var_names: A list of variable names that should not be frozen,
                         or None to freeze all the variables in the graph.
    :param output_names: Names of the relevant graph outputs.
    :param clear_devices: Remove the device directives from the graph for better portability.

    :return: The frozen graph definition.
    """
    # Fetch graph of session.
    graph = session.graph

    # Set graph as default graph and freeze graph.
    with graph.as_default():
        # Filter variable names which should be frozen.
        freeze_var_names = list(set(v.op.name for v in tf.global_variables()).difference(keep_var_names or []))
        # Set output names.
        output_names = output_names or []
        output_names += [v.op.name for v in tf.global_variables()]
        # Set input graph.
        input_graph_def = graph.as_graph_def()
        # Clear devices if not None.
        if clear_devices:
            for node in input_graph_def.node:
                node.device = ""

        # Convert variables to constants and freeze the graph.
        frozen_graph = convert_variables_to_constants(session, input_graph_def,
                                                      output_names, freeze_var_names)

        return frozen_graph


def convert_model(input_file_path, target_fold, target_file_name):
    """Convert Keras .h5 model file into Tensorflow .pb model file.

    Freeze session and convert Keras model into Tensorflow model.

    :param input_file_path: The input file path of Keras model file, in .h5.
    :param target_fold: The target fold where the Tensorflow model file is saved in.
    :param target_file_name: The output file name of Tensorflow model file, in .pb.

    :return: None. The output file has been written in function.
    """
    # Load Keras model.
    K.set_learning_phase(0)
    net_model = load_model(input_file_path)
    # Record input and output nodes' names.
    logging.info('The input node name is : %s, and the output node name is : %s' %
                 (net_model.input.name, net_model.output.name))

    # Create session and freeze graph, then save frozen graph into model file.
    output_fold = target_fold + '/tensorflow_model/'
    frozen_graph = freeze_session(K.get_session(), output_names=[net_model.output.op.name])
    graph_io.write_graph(frozen_graph, output_fold, target_file_name, as_text=False)
    logging.info('Saved the constant graph (ready for inference) at %s' %
                 osp.join(output_fold, target_file_name))


def main():
    """The main function to convert Keras model into TensorFlow model.

    :return: Write the TensorFlow model into .pb file.
    """
    # Parse the argument and read into the date.
    # ------------------------------------------
    parser = argparse.ArgumentParser(description='This is script to convert Keras model file.')
    parser.add_argument('-p', required=True, help='The target product.')
    args = parser.parse_args()
    # path = args.p

    # Define related file path, as well as file name.
    input_fold = 'THE/PATH/TO/INPUT/FOLD'
    weight_file = 'XXX.h5'
    output_graph_name = 'XXX.pb'
    output_fold = input_fold + 'tensorflow_model/'
    # Combine path with file name.
    if not os.path.isdir(output_fold):
        os.mkdir(output_fold)
    weight_file_path = osp.join(input_fold, weight_file)

    # Load Keras model.
    K.set_learning_phase(0)
    net_model = load_model(weight_file_path)
    # Record input and output nodes' names.
    logging.info('The input node name is : %s, and the output node name is : %s' %
                 (net_model.input.name, net_model.output.name))

    # Create session and freeze graph, then save frozen graph into model file.
    frozen_graph = freeze_session(K.get_session(), output_names=[net_model.output.op.name])
    graph_io.write_graph(frozen_graph, output_fold, output_graph_name, as_text=False)
    logging.info('Saved the constant graph (ready for inference) at %s' %
                 osp.join(output_fold, output_graph_name))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Import necessary packages.
import argparse
import logging
import os

from config import output_log

# Create logger for debugging.
output_log.init_log('./log/converter')


def convert_to_num(ip_address):
    """Convert ip into number.

    Convert ip address into 32-bit integer number.

    :param ip_address: The ip address, eg. 127.125.5.1.

    :return: The 32-bit number converted from ip address.
    """
    # Iterate through ip.
    number = 0
    part_num = 0
    part_str = ''
    for i in xrange(len(ip_address)):
        num = len(ip_address) - i - 1
        if ip_address[num] == '.' or num == 0:
            if num == 0:
                part_str = ip_address[num] + part_str
            if len(part_str.strip()) > 4:
                logging.error('The ip address %s has wrong length in some parts.\n' % ip_address)
                return 'Error'
            try:
                # Add number.
                number = number + int(part_str.strip()) * 2**(part_num * 8)
            except ValueError as e:
                logging.error('The ip address %s is in error: %s.\n' % (ip_address, str(e)))
                return 'Error'
            if int(part_str.strip()) > 255 or int(part_str.strip()) < 0:
                logging.error('The ip address %s has abnormal number out of bound 0~255.\n' % ip_address)
                return 'Error'
            part_num += 1
            part_str = ''
        else:
            part_str = ip_address[num] + part_str

    # Check part number.
    if part_num != 4:
        logging.error('The ip address %s is in wrong length.\n' % ip_address)
        return 'Error'

    return number


def main():
    """The main function to convert ip address into 32-bit integer.

    :return: Save result into text file.
    """
    # Parse the argument and read into the ip address.
    # ------------------------------------------
    parser = argparse.ArgumentParser(description='This is script to convert ip address into integer.')
    parser.add_argument('-f', required=True, help='The file which saves ip addresses.')
    args = parser.parse_args()
    file_name = args.f

    # Read file and parse ip into list.
    ip_list = []
    try:
        with open(file_name, 'r') as f:
            for line in f.readlines():
                ip_list.append(line.strip())
    except IOError as e:
        logging.error('Not found the input file with error info: %s.\n' % str(e))
        return 1

    # Go through ip list and convert ip address into integer.
    file_path = 'result/ip_num_result.txt'
    directory = os.path.dirname(file_path)
    try:
        os.stat(directory)
    except OSError as e:
        logging.error('Not found the input directory with error info: %s.\n' % str(e))
        os.mkdir(directory)
    with open(file_path, 'w') as f:
        for ip in ip_list:
            num = convert_to_num(ip)
            f.writelines(str(ip) + ' : ' + str(num) + '\n')


if __name__ == '__main__':
    # Run main body of script.
    main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
import argparse
import logging
import re
import time

import config.output_log as ol


def check_url_format(url):
    """Check the format of url is normal or not.

    This function is designed to use regex to detect abnormal urls.

    :param url: The url which is tended to be checked.

    :return: The string result.
            'Normal' for normal url format.
            'Error' for abnormal url format.
    """
    # Define the regex pattern of normal url.
    pattern = r"^((https|http|ftp|rtsp|mms)?://)" \
     + "+(([0-9a-zA-Z_!~*'().&=+$%-]+: )?[0-9a-zA-Z_!~*'().&=+$%-]+@)?" \
     + "(([0-9]{1,3}\\.){3}[0-9]{1,3}" \
     + "|" \
     + "([0-9a-zA-Z_!~*'()-]+\\.)*" \
     + "([0-9a-zA-Z][0-9a-zA-Z-]{0,61})?[0-9a-zA-Z]\\." \
     + "[a-zA-Z]{2,6})" \
     + "(:[0-9]{1,4})?" \
     + "((/?)|" \
     + "(/[0-9a-zA-Z_!~*'().;?:@&=+$,%#-]+)+/?)$"

    # Run the regex for test.
    if re.match(pattern, url):
        return 'Normal'
    else:
        return 'Error'


def detect_abnormal_url(dataframe, column_name):
    """Check the pattern of urls.

    Given column name of urls, check urls' pattern and return column of results.

    :param dataframe: The input dataframe in pandas.
    :param column_name: The column name of target urls, which should be converted.

    :return: The new dataframe with attached result column, consisting of 'Normal' or 'Error'.
    """
    # Check patterns of urls whether they are normal or error.
    dataframe['url_pattern'] = dataframe[column_name].apply(check_url_format, axis=0)

    return dataframe


def main():
    """The main function to check normal url format.

    :return: The output file contains urls with their results.
            The content format is 'url\tresult'.
    """
    # Input the file name.
    parser = argparse.ArgumentParser(description='This is script to check the normal url format.')
    parser.add_argument('-i', required=True, help='The input file path.')
    parser.add_argument('-o', required=True, help='The output file path.')

    # Create logger for debugging.
    ol.init_log('./log/url_check')

    # Read, parse and check the urls in file.
    args = parser.parse_args()
    file_path = args.i
    output_path = args.o

    # Total records number.
    with open(file_path) as f:
        total_records = len(f.readlines())
    with open(file_path) as f:
        # Record the starting time.
        start_time = time.time()
        logging.info('Start analyzing the url format and outputing results. '
                     'Start time is %s.' % start_time)
        count = 0.0
        with open(output_path, 'a') as fr:
            for line in f.readlines():
                count += 1  # Add count number.
                if count % 1000 == 0:
                    logging.info('The percent of running records is %s' % count/total_records)
                line = line.strip()
                result = check_url_format(line)

                # Output the content with format 'url+result'.
                fr.writelines(line + '\t' + result + '\n')

    # Record ending time.
    end_time = time.time()
    logging.info('End analyzing the url format and outputing results. '
                 'End time is %s. \nAnd the duration is %s' % (end_time, end_time-start_time))


if __name__ == '__main__':
    main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
# Import necessary packages.
import os
import logging
import logging.handlers


def init_log(log_path, level=logging.INFO, when="D", backup=7,
             formate="%(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s",
             datefmt="%m-%d %H:%M:%S"):
    """
    init_log - initialize log module

    Args:
        log_path: Log file path prefix.
            Log data will go to two files: log_path.log and log_path.log.wf
            Any non-exist parent directories will be created automatically
        level: Msg above the level will be displayed
            DEBUG < INFO < WARNING < ERROR < CRITICAL the default value is logging.INFO
        when: How to split the log file by time interval
            'S' : Seconds
            'M' : Minutes
            'H' : Hours
            'D' : Days
            'W' : Week day
            default value: 'D'
        formate: Format of the log
            default format:
            %(levelname)s: %(asctime)s: %(filename)s:%(lineno)d * %(thread)d %(message)s
            INFO: 12-09 18:02:42: log.py:40 * 139814749787872 HELLO WORLD
        backup: How many backup file to keep
            default value: 7
        datefmt: The format of datetime.

    Raises:
        OSError: Fail to create log directories
        IOError: Fail to open log file
    """
    formatter = logging.Formatter(formate, datefmt)
    logger = logging.getLogger()
    logger.setLevel(level)

    directory = os.path.dirname(log_path)
    if not os.path.isdir(directory):
        os.makedirs(directory)

    handler = logging.handlers.TimedRotatingFileHandler(log_path + ".log",
                                                        when=when,
                                                        backupCount=backup)
    handler.setLevel(level)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    handler = logging.handlers.TimedRotatingFileHandler(log_path + ".log.wf",
                                                        when=when,
                                                        backupCount=backup)
    handler.setLevel(logging.WARNING)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    # Add handler for console printing.
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    logger.addHandler(ch)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Import necessary packages.
import argparse
import datetime


def date_convert(date, days):
    """Increase or decrease date by days.

    It's designed for conveniently increasing or decreasing date.

    :param date: The date format is as followings:
                '2018-07-10' or '20180710' or '2018/07/10'
    :param days: Numerical value. Positive or negative.

    :return: Converted date in the same format of input date.
    """
    # Convert date into same formate.
    if '-' in date:
        delimiter = '-'
        date = date.replace('-', '')
    elif '/' in date:
        delimiter = '/'
        date = date.replace('/', '')
    else:
        delimiter = ''

    # Decrease or increase date.
    start_date = datetime.datetime.strptime(date, '%Y%m%d')
    delta = datetime.timedelta(days=days)
    end_date = start_date + delta

    # Convert end date into suitable format.
    end_date = end_date.strftime('%Y%m%d')
    output_date = end_date[0:4] + delimiter + end_date[4:6] + delimiter + end_date[6:8]

    return output_date


def main():
    """The main function to increase date or decrease date.

    :return: Output converted date.
    """
    # Parse the argument and read into the date.
    # ------------------------------------------
    parser = argparse.ArgumentParser(description='This is script to increase date or decrease date.')
    parser.add_argument('-d', required=True, help='The input date.')
    parser.add_argument('-n', required=True, help='The days to increase or decrease.')
    args = parser.parse_args()
    date = args.d
    days = args.n

    # Convert input date to output date, according to days.
    output_date = date_convert(date, int(days))

    print output_date


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Import necessary packages.
from __future__ import print_function
from datetime import date
import os
from setuptools import find_packages
from setuptools import setup

# Define project dependent variable.
# -----------------------------------------
NAME = "pystk"  # Your package name
GITHUB_USERNAME = "AxsPlayer"  # Your GitHub user name
VERSION = __import__(NAME).__version__  # Version of package.
AUTHOR = "AxsPlayer"
AUTHOR_EMAIL = "axsplyer@126.com"
MAINTAINER = "AxsPlayer"
MAINTAINER_EMAIL = "axsplyer@126.com"
LONG_DESCRIPTION_CONTENT_TYPE = "text/markdown"
PACKAGES = [NAME] + ["%s.%s" % (NAME, i) for i in find_packages(NAME)]  # Include all sub packages in package directory.
INCLUDE_PACKAGE_DATA = True  # Include everything in package directory.
PACKAGE_DATA = {"": ["*.*"], }
repository_name = os.path.basename(os.getcwd())  # The project directory name is the GitHub repository name.
URL = "https://github.com/{0}/{1}".format(GITHUB_USERNAME, repository_name)  # Project Url
github_release_tag = str(date.today())  # Use todays date as GitHub release tag
DOWNLOAD_URL = "https://github.com/{0}/{1}/tarball/{2}".format(
    GITHUB_USERNAME, repository_name, github_release_tag)  # Source code download url
CLASSIFIERS = ['License :: OSI Approved :: Apache Software License',
               'Programming Language :: Python :: 2.7']
PYTHON_REQUIRES = '>=2.6, !=3.*'
# PLATFORMS = ["Windows", "MacOS", "Unix"]

# Automatically generate setup parameters
# Short description.
try:
    SHORT_DESCRIPTION = __import__(NAME).__short_description__  # GitHub Short Description
except:
    print("'__short_description__' not found in '%s.__init__.py'!" % NAME)
    SHORT_DESCRIPTION = "No short description!"
# Long description.
try:
    LONG_DESCRIPTION = open("README.md", "rb").read().decode("utf-8")  # README.rst???
except:
    LONG_DESCRIPTION = "No long description!"
# License.
try:
    LICENSE = __import__(NAME).__license__
except:
    print("'__license__' not found in '%s.__init__.py'!" % NAME)
    LICENSE = ""
# Requirement.
try:
    f = open("requirements.txt", "rb")
    REQUIRES = [i.strip() for i in f.read().decode("utf-8").split("\n")]
except:
    print("'requirements.txt' not found!")
    REQUIRES = list()

# Setup all the settings.
# -----------------------
setup(
    name=NAME,
    description=SHORT_DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=MAINTAINER,
    maintainer_email=MAINTAINER_EMAIL,
    packages=PACKAGES,
    include_package_data=INCLUDE_PACKAGE_DATA,
    package_data=PACKAGE_DATA,
    url=URL,
    download_url=DOWNLOAD_URL,
    classifiers=CLASSIFIERS,
    # platforms=PLATFORMS,
    license=LICENSE,
    install_requires=REQUIRES,
    python_requires=PYTHON_REQUIRES,
)

"""
Appendix
--------
::
Frequent used classifiers List = [
    "Development Status :: 1 - Planning",
    "Development Status :: 2 - Pre-Alpha",
    "Development Status :: 3 - Alpha",
    "Development Status :: 4 - Beta",
    "Development Status :: 5 - Production/Stable",
    "Development Status :: 6 - Mature",
    "Development Status :: 7 - Inactive",
    "Intended Audience :: Customer Service",
    "Intended Audience :: Developers",
    "Intended Audience :: Education",
    "Intended Audience :: End Users/Desktop",
    "Intended Audience :: Financial and Insurance Industry",
    "Intended Audience :: Healthcare Industry",
    "Intended Audience :: Information Technology",
    "Intended Audience :: Legal Industry",
    "Intended Audience :: Manufacturing",
    "Intended Audience :: Other Audience",
    "Intended Audience :: Religion",
    "Intended Audience :: Science/Research",
    "Intended Audience :: System Administrators",
    "Intended Audience :: Telecommunications Industry",
    "License :: OSI Approved :: BSD License",
    "License :: OSI Approved :: MIT License",
    "License :: OSI Approved :: Apache Software License",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
    "Natural Language :: English",
    "Natural Language :: Chinese (Simplified)",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: MacOS",
    "Operating System :: Unix",

    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 2 :: Only",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3 :: Only",
]


CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Natural Language :: English",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: MacOS",
    "Operating System :: Unix",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
]

"""


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Import necessary packages.
import tensorflow as tf


def convert_model(pb_file_path, ckpt_file_path):
    """Convert .pb model file into .ckpt model file.

    In Tensorflow, there are several kinds of model files. One mission
    is to convert .pb model file into .ckpt model file.

    :param pb_file_path: The file path of .pb model.
    :param ckpt_file_path: The file path of .ckpt model.

    :return: None. The result model file is saved in function.
    """
    # Create graph in Tensorflow and start converting.
    with tf.Graph().as_default():
        output_graph_def = tf.GraphDef()
        # Start importing model.
        with open(pb_file_path, "rb") as f:
            output_graph_def.ParseFromString(f.read())
            # tensors = tf.import_graph_def(output_graph_def, name="test")
        saver = tf.train.Saver()
        # Create Tensorflow's session.
        with tf.Session() as sess:
            # Initialize variable.
            init = tf.global_variables_initializer()
            sess.run(init)
            op = sess.graph.get_operations()
            # Print operation.
            for m in op:
                print(m.values())
            # Save model into .ckpt file.
            saver.save(sess, ckpt_file_path)
